
import os
import random

from .forms import UploadImageForm
from django.core.cache import cache

from django.http import FileResponse

from django.shortcuts import render, redirect
from django.conf import settings
from django.http import HttpResponseRedirect
from django.urls import reverse
from urllib.parse import unquote

import pyheif
from PIL import Image
import magic

# def view_images(request):
#     photos = Photo.objects.all()
#     return render(request, "view_images.html", {"photos": photos})



# # BEGIN SHOW_RANDOM_IMAGE ALTERNATE ##################################################

# # Global variables to store the shuffled list and the time it was shuffled
# shuffled_image_files = []
# last_shuffle_time = datetime.now()

# def show_random_image(request):
#     global shuffled_image_files
#     global last_shuffle_time

#     # Get the current time
#     current_time = datetime.now()

#     # Check if an hour has passed since the last shuffle
#     if not shuffled_image_files or current_time - last_shuffle_time > timedelta(hours=1):
#         # Get a list of all image files
#         image_dir = os.path.join(settings.MEDIA_ROOT, 'images')
#         image_files = [f for f in os.listdir(image_dir) if os.path.isfile(os.path.join(image_dir, f))]

#         # Shuffle the list of image files
#         random.shuffle(image_files)

#         # Store the shuffled list and the current time
#         shuffled_image_files = image_files
#         last_shuffle_time = current_time

#     # Select a random image file from the shuffled list
#     random_image_file = random.choice(shuffled_image_files)

#     # Create the full file path
#     file_path = os.path.join(image_dir, random_image_file)

#     # Return the image file to the browser
#     return FileResponse(open(file_path, 'rb'), content_type='image/jpeg')

# # END SHOW_RANDOM_IMAGE ALTERNATE ##################################################


def show_image(request):
    """
    View function that returns a somewhat random image from the 'my_photos' directory.
    There is a custom algorithm to select the image.

    Args:
        request: The HTTP request object.

    Returns:
        A FileResponse object containing the random image file.
    """
    # Get a list of all image files
    image_dir = os.path.join(settings.MEDIA_ROOT, "my_photos")
    cache_image_files = cache.get('cache_image_files')

    current_image_files = [f for f in os.listdir(image_dir) if os.path.isfile(os.path.join(image_dir, f))]
    current_count = len(current_image_files)


    if not cache_image_files or len(cache_image_files) != current_count:
        random.shuffle(current_image_files)
        cache_image_files = current_image_files
        cache.set('cache_image_files', current_image_files, 86400)  # Cache for 24 hours

    current_index = cache.get('current_index', 0)
    image_file = cache_image_files[current_index]

    print('serving image:', image_file)

    # Create the full file path
    file_path = os.path.join(image_dir, image_file)

    # Update the current index
    current_index = (current_index + 1) % len(cache_image_files)
    cache.set('current_index', current_index)
    print('current_index:', current_index)

    # Return the image file to the browser
    return FileResponse(open(file_path, "rb"), content_type="image/jpeg")

def show_random_image(request):
    # Get a list of all image files
    image_dir = os.path.join(settings.MEDIA_ROOT, "my_photos")
    image_files = [f for f in os.listdir(image_dir) if os.path.isfile(os.path.join(image_dir, f))]

    random.shuffle(image_files)

    # Select a random image file
    random_image_file = random.choice(image_files)

    print(random_image_file)

    # Create the full file path
    file_path = os.path.join(image_dir, random_image_file)

    # Return the image file to the browser
    return FileResponse(open(file_path, "rb"), content_type="image/jpeg")



def upload_image(request):
    """ ?? Display the form to upload images and process the uploaded images."""
    if request.method == "POST":
        form = UploadImageForm(request.POST, request.FILES)
        if form.is_valid():
            # Get the uploaded image files
            image_files = request.FILES.getlist("images")

            # Process each uploaded image
            image_urls = []
            for image_file in image_files:
                # Save the image file to a designated directory
                image_path = handle_uploaded_image(image_file)

                # Check if the file is in HEIC format
                file_type = get_file_type(image_path)
                if 'heic' in file_type:
                    # Convert the HEIC image to JPEG
                    image_path = convert_heic_to_jpeg(image_path)

                # Construct the URL of the uploaded image
                relative_path = os.path.relpath(image_path, settings.MEDIA_ROOT)
                image_url = os.path.join(settings.MEDIA_URL, relative_path)
                image_urls.append(image_url)

            # Optionally, do something with the image_urls, such as displaying them on a page or storing them in a session
            return render(request, "upload_success.html", {"image_urls": image_urls})
    else:
        form = UploadImageForm()
    return render(request, "upload_image.html", {"form": form})


def handle_uploaded_image(image_file):
    """ ?? Save the uploaded image to a designated directory and return the file path."""

    # Determine the path to save the uploaded image
    print(settings.MEDIA_ROOT)
    upload_dir = settings.MEDIA_ROOT + "/my_photos"  # Change this to the desired directory
    image_path = os.path.join(upload_dir, image_file.name)

    # Save the uploaded image to the designated directory
    with open(image_path, "wb") as destination:
        for chunk in image_file.chunks():
            destination.write(chunk)

    return image_path

def get_file_type(file_path):
    """ Return the MIME type of the file. """
    mime = magic.Magic(mime=True)
    file_type = mime.from_file(file_path)
    print("uploaded file type:", file_type)
    return file_type

def convert_heic_to_jpeg(heic_path):
    """ Convert the HEIC image to JPEG format and return the path to the JPEG file. """

    print("Converting HEIC file path:", heic_path)
    # Read the HEIC file
    heif_file = pyheif.read(heic_path)

    # Convert the HEIC file to a PIL Image object
    image = Image.frombytes(
        heif_file.mode,
        heif_file.size,
        heif_file.data,
        "raw",
        heif_file.mode,
        heif_file.stride,
    )

    # Change the file extension to .jpg
    jpeg_path = os.path.splitext(heic_path)[0] + '.jpg'

    # Save the image in JPEG format
    image.save(jpeg_path, format='JPEG')

    # Delete the original HEIC file
    os.remove(heic_path)

    return jpeg_path

def display_jpegs(request):
    # Path to the directory containing JPEG images
    directory = os.path.join(settings.BASE_DIR, "media/my_photos")

    # Get a list of all files in the directory
    files = os.listdir(directory)

    # Filter the list to include only JPEG files
    # jpegs = [file for file in files if file.lower().endswith(".jpeg")]
    images = [file for file in files]

    # Generate URLs for the JPEG images
    image_urls = [os.path.join(settings.MEDIA_URL, "my_photos", image) for image in images]

    return render(request, "display_jpegs.html", {"image_urls": image_urls})

def delete_image(request, image_url):
    # Unquote the URL to get the original string
    image_url = unquote(image_url)

    # Extract the image name from the URL
    image_name = image_url.split('/')[-1]

    # Path to the directory containing JPEG images
    directory = os.path.join(settings.BASE_DIR, "media/my_photos")

    # Full path to the image
    image_path = os.path.join(directory, image_name)

    # Check if the file exists and delete it
    if os.path.isfile(image_path):
        os.remove(image_path)

    # Redirect back to the display_jpegs view
    return HttpResponseRedirect(reverse('display_jpegs'))



def root_redirect(request):
    return redirect('display_jpegs')  # 'display_jpegs' is the name of the view